/*

    this source file is a part of 
    lp_parse - a preprocessor for MILP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

 */

#ifndef HASH_H
#include "hash.h"
#endif
#define LP_INDEX_H

struct index {
  ITEM_DATA;
  int has_value;
  int value;
  int has_bounds; /* indicates that bounds are as expressions */
  int min;
  int max;
  char *smin;
  char *smax;
};

#define INDEX struct index

int init_index(INDEX *index, char *name);

INDEX *new_index(char *name);

void delete_index(INDEX *index);

void dispose_index(INDEX *index);

/* 
 * the new index is the new root, this is because it is easier to loop
 * indexlist that is arranged in this way 
*/
INDEX *add_index(INDEX *root, INDEX *i);

void remove_index(INDEX **root, int n);

void delete_indexes(INDEX **root);

INDEX *search_index(INDEX *root, char *name);

int init_index_loop(INDEX *root, int n);

int loop_indexes(INDEX *root, int n);

void print_indexes(INDEX *root);


/*---------------------------------------------------------------------------*/


struct constant {
  ITEM_DATA;
  void *next_in_list;
  int has_value;
  double value;	  
};

#define CONSTANT struct constant

int init_constant(CONSTANT *constant, char *name);

CONSTANT *new_constant(char *name);

void delete_constant(CONSTANT *constant);

void dispose_constant(CONSTANT *constant);

struct constant_map {
  CONSTANT *list;
  CONSTANT *last_in_list;
  HASH *hash;
};

#define CMAP struct constant_map

int hash_constants(CMAP *map);

void add_constant(CMAP *map, CONSTANT *c);

CONSTANT *search_constant(CMAP *map, char *name);

int update_constant(CMAP *map, char *name, double value);

void print_constants(CMAP *map, int values);





