/*

    this source file is a part of 
    lp_parse - a preprocessor for MIP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "lp_out.h"

char operator_buffer = '\0';
double number_buffer;
int number_buffer_used = FALSE;
int holding_number = FALSE;
char calculator_buffer;

void operator_out(XFILE *f) {
  if (operator_buffer) {
    switch (operator_buffer) {
    case ';':;
    case '+': 
      if (f->buffer)
	f->buf_size += sprintf(f->buffer+f->buf_size, "%c\n", operator_buffer);
      else
	fprintf(f->f, "%c\n", operator_buffer); 
      break;
    default: 
      if (f->buffer)
	f->buf_size += sprintf(f->buffer+f->buf_size, "%c", operator_buffer);
      else
	fprintf(f->f, "%c", operator_buffer);
    }
    operator_buffer = '\0';
  }
}

void number_out(XFILE *f) {
  if (number_buffer_used) {
    if ((number_buffer < 0) && 
	((operator_buffer == '+') || 
	 (operator_buffer == '-')))
      operator_buffer = '\n';
    operator_out(f);
    if (f->buffer)
	f->buf_size += sprintf(f->buffer+f->buf_size, "%f\n", number_buffer);
      else
	fprintf(f->f, "%f", number_buffer);
    number_buffer_used = FALSE;
  }
}

void streamer_flush(XFILE *f) {
  if (holding_number) {
    number_out(f);
    operator_buffer = calculator_buffer;
    holding_number = FALSE;
  }
  operator_out(f);
}

void stream_out_token(XFILE *f, char *token) {
  streamer_flush(f);
  if (f->buffer)
    f->buf_size += sprintf(f->buffer+f->buf_size, "%s\n", token);
  else	
    fprintf(f->f, "%s", token);
}

void stream_out_char(XFILE *f, char c) {
  if (((c == '*') || (c == '/')) && number_buffer_used) {
    holding_number = TRUE;
    calculator_buffer = c;
  } else {
    number_out(f);
    operator_buffer = c;
  }
}

void stream_out_number(XFILE *f, double x) {
  if (holding_number) {
    switch (calculator_buffer) {
    case '*': number_buffer *= x; break;
    case '/': number_buffer /= x; break;
    }
    holding_number = FALSE;
  } else {
    number_buffer = x;
    number_buffer_used = TRUE;
  }
}

